import { createWriteStream } from 'fs';
import { readFile } from 'fs/promises';
import { ADAPTER_ID } from './common';

export interface CreateAdapterParams {
  type: typeof ADAPTER_ID;
  path: string;
}

class Adapter<Entry = unknown, Query extends Record<string, unknown> = Record<string, unknown>> {
  private data?: Entry[];

  private readonly path: string;

  constructor(path: string) {
    this.path = path;
  }

  async connect() {
    const buf = await readFile(this.path);
    const str = buf.toString('utf-8');
    const json = str.split('\n');
    this.data = json.map((j) => JSON.parse(j) as Entry);
  }

  async createIngester() {
    return Promise.resolve(createWriteStream(this.path));
  }

  async search(query: Query): Promise<Entry[]> {
    if (!this.data) {
      throw new Error('Adapter not initialized.');
    }

    const results = this.data.filter((dd) => (
      Object.entries(query).reduce(
        (prevFilterResult, [key, value]) => {
          const d = dd as Record<string, unknown>;
          if (typeof d[key] === 'string') {
            const dString = d[key] as string;
            return (
              prevFilterResult
              && dString.toLowerCase().includes((value as string).toString().toLowerCase())
            );
          }
          if (typeof d[key] === 'number') {
            return prevFilterResult && Number(d[key]) === Number(value);
          }
          return prevFilterResult && d[key] === value;
        },
        true,
      )
    ));

    return Promise.resolve(results);
  }
}

export const createAdapter = async (params: Omit<CreateAdapterParams, 'type'>) => {
  const { path } = params;

  const adapter = new Adapter(path);
  await adapter.connect();
};

export interface CreateIngesterParams {
  type: typeof ADAPTER_ID;
  path: string;
}

export const createIngester = async (params: Omit<CreateIngesterParams, 'type'>) => {
  const { path } = params;

  const adapter = new Adapter(path);
  return adapter.createIngester();
};
